@@ -382,109 +382,113 @@ public class PdfCopy extends PdfWriter {
 				if( annots!= null && annots.isArray() ) {
 					ArrayList annots_arr= annots.getArrayList();
 					for( int ii= 0; ii< annots_arr.size(); ++ii ) {
-						PdfIndirectReference annot_ref= (PdfIndirectReference)annots_arr.get(ii);
-						if( annot_ref!= null && annot_ref.isIndirect() ) {
-							PdfDictionary annot= (PdfDictionary)reader.getPdfObject(annot_ref);
-							if( annot!= null && annot.isDictionary() ) {
-								PdfName subtype= (PdfName)reader.getPdfObject(annot.get(PdfName.SUBTYPE));
-								if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
-									// we have a form field
+						// an annotation may be direct or indirect; ours must be indirect
+						PdfObject annot_obj= (PdfObject)annots_arr.get(ii);
+						if( annot_obj!= null && annot_obj.isIndirect() ) {
+							PdfIndirectReference annot_ref= (PdfIndirectReference)annot_obj;
+							if( annot_ref!= null ) {
+								PdfDictionary annot= (PdfDictionary)reader.getPdfObject(annot_ref);
+								if( annot!= null && annot.isDictionary() ) {
+									PdfName subtype= (PdfName)reader.getPdfObject(annot.get(PdfName.SUBTYPE));
+									if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
+										// we have a form field
 
-									// get its full name
-									//
-									String full_name= ""; // construct a full name from partial names using '.', e.g.: foo.bar.
-									String top_name= "";
-									PdfString tt= (PdfString)reader.getPdfObject(annot.get(PdfName.T));
-									if( tt!= null && tt.isString() ) {
-										top_name= tt.toString();
-									}
-									//
-									// dig upwards, parent-wise; replace annot as we go with the
-									// top-most form field dictionary
-									PdfIndirectReference parent_ref= 
-										(PdfIndirectReference)annot.get(PdfName.PARENT);
-									while( parent_ref!= null && parent_ref.isIndirect() )
-										{
-											annot_ref= parent_ref;
-											annot= (PdfDictionary)reader.getPdfObject(annot_ref);
-											parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
-
-											tt= (PdfString)reader.getPdfObject(annot.get(PdfName.T));
-											if( tt!= null && tt.isString() ) {
-												if( top_name.length()!= 0 ) {
-													full_name+= top_name;
-													full_name+= ".";
-												}
-												top_name= tt.toString();
-											}
+										// get its full name
+										//
+										String full_name= ""; // construct a full name from partial names using '.', e.g.: foo.bar.
+										String top_name= "";
+										PdfString tt= (PdfString)reader.getPdfObject(annot.get(PdfName.T));
+										if( tt!= null && tt.isString() ) {
+											top_name= tt.toString();
 										}
+										//
+										// dig upwards, parent-wise; replace annot as we go with the
+										// top-most form field dictionary
+										PdfIndirectReference parent_ref= 
+											(PdfIndirectReference)annot.get(PdfName.PARENT);
+										while( parent_ref!= null && parent_ref.isIndirect() )
+											{
+												annot_ref= parent_ref;
+												annot= (PdfDictionary)reader.getPdfObject(annot_ref);
+												parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
+
+												tt= (PdfString)reader.getPdfObject(annot.get(PdfName.T));
+												if( tt!= null && tt.isString() ) {
+													if( top_name.length()!= 0 ) {
+														full_name+= top_name;
+														full_name+= ".";
+													}
+													top_name= tt.toString();
+												}
+											}
 									
-									// once we have seen a top-level field parent, we wave
-									// it through and assume that it harbors no illegal field duplicates;
-									// this is good, because sometimes authors want a single field
-									// represented by more than one annotation on the page; this logic
-									// respects that programming
-									//
-									//System.out.println( full_name+ top_name+ "." ); // debug
-									if( readerData.allNames.contains( top_name ) )
-										{ // a parent we have seen or created in this reader
-											this.fullFormFieldNames.add( full_name+ top_name+ "." ); // tally
-										}
-									else {
-										if( this.fullFormFieldNames.contains( full_name+ top_name+ "." ) ) {
-											// insert new, top-most parent
+										// once we have seen a top-level field parent, we wave
+										// it through and assume that it harbors no illegal field duplicates;
+										// this is good, because sometimes authors want a single field
+										// represented by more than one annotation on the page; this logic
+										// respects that programming
+										//
+										//System.out.println( full_name+ top_name+ "." ); // debug
+										if( readerData.allNames.contains( top_name ) )
+											{ // a parent we have seen or created in this reader
+												this.fullFormFieldNames.add( full_name+ top_name+ "." ); // tally
+											}
+										else {
+											if( this.fullFormFieldNames.contains( full_name+ top_name+ "." ) ) {
+												// insert new, top-most parent
 
-											// name for new parent
-											int new_parent_name_ii= 1;
-											String new_parent_name= Integer.toString( new_parent_name_ii );
-											while( this.fullFormFieldNames.contains( full_name+ top_name+ "."+ new_parent_name+ "." ) ||
-												   this.topFormFieldNames.contains( new_parent_name ) &&
-												   !readerData.newNamesKids.containsKey( new_parent_name ) )
-												{
-													new_parent_name= Integer.toString( ++new_parent_name_ii );
+												// name for new parent
+												int new_parent_name_ii= 1;
+												String new_parent_name= Integer.toString( new_parent_name_ii );
+												while( this.fullFormFieldNames.contains( full_name+ top_name+ "."+ new_parent_name+ "." ) ||
+													   this.topFormFieldNames.contains( new_parent_name ) &&
+													   !readerData.newNamesKids.containsKey( new_parent_name ) )
+													{
+														new_parent_name= Integer.toString( ++new_parent_name_ii );
+													}
+
+												PdfIndirectReference new_parent_ref= null;
+												PdfArray new_parent_kids= null;
+												//
+												if( readerData.newNamesKids.containsKey( new_parent_name ) ) {
+													// a new parent we already created
+													new_parent_ref= (PdfIndirectReference)
+														readerData.newNamesRefs.get( new_parent_name );
+													new_parent_kids= (PdfArray)
+														readerData.newNamesKids.get( new_parent_name );
+												}
+												else { // create a new parent using this name
+													PdfDictionary new_parent= new PdfDictionary();
+													new_parent_ref= reader.getPRIndirectReference( new_parent );
+													new_parent.put( PdfName.T, new PdfString( new_parent_name ) );
+											
+													new_parent_kids= new PdfArray();
+													PdfIndirectReference new_parent_kids_ref= 
+														reader.getPRIndirectReference( new_parent_kids );
+													new_parent.put(PdfName.KIDS, new_parent_kids_ref);
+
+													// tally new parent
+													readerData.newNamesRefs.put( new_parent_name, new_parent_ref );
+													readerData.newNamesKids.put( new_parent_name, new_parent_kids );
+													readerData.allNames.add( new_parent_name );
+													this.topFormFieldNames.add( new_parent_name );
 												}
 
-											PdfIndirectReference new_parent_ref= null;
-											PdfArray new_parent_kids= null;
-											//
-											if( readerData.newNamesKids.containsKey( new_parent_name ) ) {
-												// a new parent we already created
-												new_parent_ref= (PdfIndirectReference)
-													readerData.newNamesRefs.get( new_parent_name );
-												new_parent_kids= (PdfArray)
-													readerData.newNamesKids.get( new_parent_name );
+												// wire annot and new parent together
+												annot.put( PdfName.PARENT, new_parent_ref );
+												new_parent_kids.add( annot_ref ); // the new parent must point at the field, too
+
+												// tally full field name
+												this.fullFormFieldNames.add( full_name+ top_name+ "."+ new_parent_name+ "." );
 											}
-											else { // create a new parent using this name
-												PdfDictionary new_parent= new PdfDictionary();
-												new_parent_ref= reader.getPRIndirectReference( new_parent );
-												new_parent.put( PdfName.T, new PdfString( new_parent_name ) );
-											
-												new_parent_kids= new PdfArray();
-												PdfIndirectReference new_parent_kids_ref= 
-													reader.getPRIndirectReference( new_parent_kids );
-												new_parent.put(PdfName.KIDS, new_parent_kids_ref);
+											else {
+												// tally parent
+												readerData.allNames.add( top_name );
+												this.topFormFieldNames.add( top_name );
 
-												// tally new parent
-												readerData.newNamesRefs.put( new_parent_name, new_parent_ref );
-												readerData.newNamesKids.put( new_parent_name, new_parent_kids );
-												readerData.allNames.add( new_parent_name );
-												this.topFormFieldNames.add( new_parent_name );
+												// tally full field name
+												this.fullFormFieldNames.add( full_name+ top_name+ "." );
 											}
-
-											// wire annot and new parent together
-											annot.put( PdfName.PARENT, new_parent_ref );
-											new_parent_kids.add( annot_ref ); // the new parent must point at the field, too
-
-											// tally full field name
-											this.fullFormFieldNames.add( full_name+ top_name+ "."+ new_parent_name+ "." );
-										}
-										else {
-											// tally parent
-											readerData.allNames.add( top_name );
-											this.topFormFieldNames.add( top_name );
-
-											// tally full field name
-											this.fullFormFieldNames.add( full_name+ top_name+ "." );
 										}
 									}
 								}
@@ -509,27 +513,31 @@ public class PdfCopy extends PdfWriter {
 				if( annots!= null && annots.isArray() ) {
 					ArrayList annots_arr= annots.getArrayList();
 					for( int ii= 0; ii< annots_arr.size(); ++ii ) {
-						PdfIndirectReference annot_ref= (PdfIndirectReference)annots_arr.get(ii);
-						if( annot_ref!= null && annot_ref.isIndirect() ) {
-							PdfDictionary annot= (PdfDictionary)reader.getPdfObject(annot_ref);
-							if( annot!= null && annot.isDictionary() ) {
-								PdfName subtype= (PdfName)reader.getPdfObject(annot.get(PdfName.SUBTYPE));
-								if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
-									// we have a form field
+						// an annotation may be direct or indirect; ours must be indirect
+						PdfObject annot_obj= (PdfObject)annots_arr.get(ii);
+						if( annot_obj!= null && annot_obj.isIndirect() ) {
+							PdfIndirectReference annot_ref= (PdfIndirectReference)annots_arr.get(ii);
+							if( annot_ref!= null ) {
+								PdfDictionary annot= (PdfDictionary)reader.getPdfObject(annot_ref);
+								if( annot!= null && annot.isDictionary() ) {
+									PdfName subtype= (PdfName)reader.getPdfObject(annot.get(PdfName.SUBTYPE));
+									if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
+										// we have a form field
 
-									// dig upwards, parent-wise
-									PdfIndirectReference parent_ref= 
-										(PdfIndirectReference)annot.get(PdfName.PARENT);
-									while( parent_ref!= null && parent_ref.isIndirect() ) {
-										annot_ref= parent_ref;
-										annot= (PdfDictionary)reader.getPdfObject(annot_ref);
-										parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
-									}
+										// dig upwards, parent-wise
+										PdfIndirectReference parent_ref= 
+											(PdfIndirectReference)annot.get(PdfName.PARENT);
+										while( parent_ref!= null && parent_ref.isIndirect() ) {
+											annot_ref= parent_ref;
+											annot= (PdfDictionary)reader.getPdfObject(annot_ref);
+											parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
+										}
 								
-									RefKey annot_key= new RefKey(annot_ref);
-									IndirectReferences annot_iRef= (IndirectReferences)indirects.get(annot_key);
-									PdfAcroForm acroForm= this.getAcroForm();
-									acroForm.addDocumentField( annot_iRef.getRef() );
+										RefKey annot_key= new RefKey(annot_ref);
+										IndirectReferences annot_iRef= (IndirectReferences)indirects.get(annot_key);
+										PdfAcroForm acroForm= this.getAcroForm();
+										acroForm.addDocumentField( annot_iRef.getRef() );
+									}
 								}
 							}
 						}
